clear;
clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%             structural parameters of the economy            %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
alpha   = 0.90;
lambda  = 0.50;
beta    = 0.99;
gamy    = 0.50;
gamp    = 1.50;
ru      = 0.95;
su		= 0.01;
rg      = 0.97;
sg		= 0.01;
SIG		= diag(su,sg);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                 Parameters of the algorithm                 %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ny      = 1;              		% # of static equations
nb      = 1;               		% # of endogenous predetermined variables
nf      = 2;                    % # of expectational equations
nz      = 2;               		% # of shocks
nx      = nb+nf;                % Total # of endogenous states and costates
Ny      = zeros(ny,ny); 
Nx      = zeros(ny,nx); 
Nz      = zeros(ny,nz); 
Mx0     = zeros(nx,nx); 
Mx1     = zeros(nx,nx); 
My0     = zeros(nx,ny); 
My1     = zeros(nx,ny); 
Mz0     = zeros(nx,nz); 
Mz1     = zeros(nx,nz); 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                      matrix coefficients                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% i 
% 1 
%
% z y pi
% 1 2 3
%
% u g
% 1 2
%
% Taylor Rule
%
Ny(1,1)	= 1;
Nx(1,2)	= gamy;
Nx(1,3)	= gamp;
Nz(1,1)	= 1;
%
% Technical Variable
%
Mx0(1,1)	= 1;
Mx1(1,3)	= 1;
%
% IS Curve
%
Mx0(2,2)	= 1;
Mx0(2,3)	= alpha;
Mx1(2,2)	= 1;
My1(2,1)	= alpha;
Mz1(2,2)	= -1;
%
% Phillips Curve
%
Mx0(3,3)	= beta/(1+beta);
Mx1(3,1)	= -1/(1+beta);
Mx1(3,2)	= -lambda;
Mx1(3,3)	= 1;
Mz1(3,1)	= -1;

Phi         = [ru 0;0 rg];
Me          = [1 0;0 1];

model.ny    = Ny;
model.nx    = Nx;
model.nz    = Nz;
model.my0   = My0;
model.mx0   = Mx0;
model.mz0   = Mz0;
model.my1   = My1;
model.mx1   = Mx1;
model.mz1   = Mz1;
model.me    = Me;
model.phi   = Phi;
model.nb    = nb;
model.nf    = nf;
model.sig   = SIG;

sol         = sdge_solve(model);
irfu        = comp_irf(sol,[1;0],20);
irfg        = comp_irf(sol,[0;1],20);

subplot(221);plot(irfu.z(:,1));title('u');
subplot(222);plot(irfu.f(:,1));title('y');
subplot(223);plot(irfu.f(:,2));title('\pi');
subplot(224);plot(irfu.y(:,1));title('i');
pause;
close;

subplot(221);plot(irfg.z(:,2));title('g');
subplot(222);plot(irfg.f(:,1));title('y');
subplot(223);plot(irfg.f(:,2));title('\pi');
subplot(224);plot(irfg.y(:,1));title('i');
pause;
close;
