%function [MY,ME,MU_]=sims_solve(A0,A1,B,C)
clear all
%   
% Structural Parameters
%
r      = 0.02;
f0     = 1;
wb     = 0.6;
f1     = 0.2;
phi    = 1;
rho    = 0.95;
nb     = (f0-wb)/f1;

A0=[
    1 -1 0 0
    0  1 0 0
    0  0 1 0
    0 -(2+r+f1*(1+r)/phi) -(1+r)/phi 1  
];

A1=[ 
    0 0 0 0
    0 0 0 1
    0 0 rho 0
    -(1+r) 0 0 0
];

B=[0;0;1;0];
C=[0;1;0;0];
%
% First we compute the Schur decomposition A0=Q'*T*Z' and A1=Q'*S*Z'
%
tol_=1e-8;
[T S Q Z]	= qz(A0,A1);
[T S Q Z]   = qzdiv(1.0001,T,S,Q,Z);
n           = size(A0,1);
%
% We recover the diagonal of the S and T matrix in order to 
%
%	1) Determine the status of variables
%	2) compute the generalized eigenvalues
%
DS_			= diag(S);
DT_			= diag(T);
%
% Dynamic variables 
%
dyn_		= find(abs(DT_)>tol_);	    % Determine dynamic variables
MU_			= DS_(dyn_)./DT_(dyn_);		% Compute generalized eigenvalues
AMU_		= abs(MU_);				    % Compute their modulus
m			= sum(AMU_>1+tol_);			% Find the forward variables
Q1          = Q(1:n-m,:);
Q2          = Q(n-m+1:n,:);
Z1          = Z(:,1:n-m)';
Z2          = Z(:,n-m+1:n)';

[U1,D1,V1]   = svd(Q1*C);
r1           = rank(Q1*C);
[U2,D2,V2]   = svd(Q2*C);
r2           = rank(Q2*C);

Phi         = ((U2(:,1:r2)*(D2(1:r2,1:r2)\V2(:,1:r2)'))*(V1(:,1:r1)*D1(1:r1,1:r1)*U1(:,1:r1)'))';
T11         = T(1:n-m,1:n-m);
T12         = T(1:n-m,n-m+1:n);
T22         = T(n-m+1:n,n-m+1:n);

T1i         = inv(T11);
S11         = S(1:n-m,1:n-m);
S12         = T(1:n-m,n-m+1:n);
S22         = T(n-m+1:n,n-m+1:n);

Transfo     = [eye(n-m) -Phi];
MY0         = inv([Transfo*T;zeros(m,n-m) eye(m)]);
MY1         = ([Transfo*S;zeros(m,n)]);
MY          = Z*(MY0*MY1)*Z';
ME          = MY0*[Transfo*Q*B;zeros(m,size(B,2))];
ME          = Z*ME;

%
% IRF
%
nrep=20;
SHOCK=[1]; 
Y=zeros(n,nrep);
Y(:,1)=ME*SHOCK;
for i=2:nrep;
  Y(:,i)=MY*Y(:,i-1);
end
T=1:nrep;
subplot(221);plot(T,Y(3,:));
xlabel('Time','fontname','times','fontsize',12);title('Real Wage','fontname','times','fontsize',12);
set(gca,'fontname','times','fontsize',12)
subplot(222);plot(T,Y(2,:));
xlabel('Time','fontname','times','fontsize',12);title('Labor Demand','fontname','times','fontsize',12);
set(gca,'fontname','times','fontsize',12)
pause;
print -depsc2 labor2.eps;
close