clear;
clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%             structural parameters of the economy            %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
alpha   = 0.36;
delta   = 0.025;
beta    = 0.99;
gx      = 1.005;
chi 	= 1;
ra      = 0;
sa		= 0.008;
rg      = 0.97;
sg		= 0.01;
gsy     = 0.2;
%SIG		= diag(sa,sg);
SIG		= diag([sa sg]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                          steady state                       %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ysk     = (gx-beta*(1-delta))/(alpha*beta*gx);
isk     = (gx+delta-1)/gx;
isy     = isk/ysk;
csy     = 1-isy-gsy;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                 Parameters of the algorithm                 %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ny      = 4;              		% # of static equations
nb      = 1;               		% # of endogenous predetermined variables
nz      = 2;               		% # of shocks
nf      = 1;                    % # of expectational equations
nx      = nb+nf;                % Total # of endogenous states and costates
Ny      = zeros(ny,ny); 
Nx      = zeros(ny,nx); 
Nz      = zeros(ny,nz); 
Mx0     = zeros(nx,nx); 
Mx1     = zeros(nx,nx); 
My0     = zeros(nx,ny); 
My1     = zeros(nx,ny); 
Mz0     = zeros(nx,nz); 
Mz1     = zeros(nx,nz); 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                      matrix coefficients                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% y c i h
% 1 2 3 4
%
% k lb
% 1 2
%
% a g
% 1 2
%
% Output
%
Ny(1,1)	= 1;
Ny(1,4)	= alpha-1;
Nx(1,1)	= alpha;
Nz(1,1)	= -alpha;
%
% consumption
%
Ny(2,2)	= -1;
Nx(2,2)	= 1;
%
% Investment
%
Ny(3,1)	= 1;
Ny(3,2)	= -csy;
Ny(3,3)	= -isy;
Nz(3,2)	= gsy;
%
% Hours worked
%
Ny(4,1)	= -1;
Ny(4,4)	= 1+chi;
Nx(4,2)	= 1;
%
% capital
%
Mx0(1,1)	=	gx;
Mx1(1,1)	=	1-delta;
My1(1,3)	=	gx+delta-1;
Mz1(1,1)	=	delta-1;
%
% Euler
%
Mx0(2,1)	= -(gx-beta*(1-delta))/gx;
Mx0(2,2)	= 1;
Mz0(2,1)	= -beta*(1-delta)/gx;
My0(2,1)	= (gx-beta*(1-delta))/gx;
Mx1(2,2)	= 1;
%
% Shocks
%
Phi         = [ra 0;0 rg];
Me          = [1 0;0 1];

model.ny    = Ny;
model.nx    = Nx;
model.nz    = Nz;
model.my0   = My0;
model.mx0   = Mx0;
model.mz0   = Mz0;
model.my1   = My1;
model.mx1   = Mx1;
model.mz1   = Mz1;
model.me    = Me;
model.phi   = Phi;
model.nb    = nb;
model.nf    = nf;
model.sig   = SIG;
sol         = sdge_solve(model);

nrep        = 20;
irfa        = comp_irf(sol,[1;0],nrep);
irfg        = comp_irf(sol,[0;1],nrep);
Ta          = cumsum(irfa.z(:,1));
Tg          = cumsum(irfg.z(:,1));
subplot(221);plot(Ta);title('a');
subplot(222);plot(irfa.x(:,1)+[0;Ta(1:nrep-1)]);title('k');
disp('strike any key to continue')
disp(' ') 
pause;
close;
pause;
close;
subplot(221);plot(irfa.y(:,1)+Ta);title('y');
subplot(222);plot(irfa.y(:,4));title('h');
subplot(223);plot(irfa.y(:,2)+Ta);title('c');
subplot(224);plot(irfa.y(:,3)+Ta);title('i');
disp('strike any key to continue')
disp(' ') 
pause;
close;
pause;
close;

subplot(221);plot(irfg.z(:,2)+Tg);title('g');
subplot(222);plot(irfg.x(:,1)+[0;Tg(1:nrep-1)]);title('k');
disp('strike any key to continue')
disp(' ') 
pause;
close;
pause;
close;
subplot(221);plot(irfg.y(:,1)+Tg);title('y');
subplot(222);plot(irfg.y(:,4));title('h');
subplot(223);plot(irfg.y(:,2)+Tg);title('c');
subplot(224);plot(irfg.y(:,3)+Tg);title('i');
disp('strike any key to continue')
disp(' ') 
pause;
close;



nsim=1000;
long=100;
init=50;
slong=init+long;
smpl=init+1:slong;
STD=zeros(nsim,ny);
for s=1:nsim;
    randn('state',s);
    %shocks=randn(nz,slong)*sol.sig;
    shocks=sol.sig*randn(nz,slong);
    sim=sim_data(sol,shocks);
    trend=cumsum(sim.z(smpl,1));
    dy=diff(sim.y(smpl,:)+repmat(trend,1,ny));
    STD(s,:)=std(dy);
end
disp('STD')
disp(mean(STD))
