function sol=sdge_solve(model);
Ny      = model.ny;
Nx      = model.nx;
Nz      = model.nz;
Mx1     = model.mx1;
Mz1     = model.mz1;
My1     = model.my1;
Mx0     = model.mx0;
Mz0     = model.mz0;
My0     = model.my0;
Phi     = model.phi;
Me      = model.me;
nb      = model.nb;
nf      = model.nf;
ny      = size(Ny,1);
nz      = size(Nz,2);

TMPx      = Ny\Nx;    
TMPx1   = TMPx(:,1:nb);
TMPx2   = TMPx(:,nb+1:nb+nf);
TMPz    = Ny\Nz;
A       = Mx0+My0*TMPx;
B       = Mx1+My1*TMPx;
C       = Mz1+My1*TMPz-(Mz0+My0*TMPz)*Phi;

[S,T,Q,Z] = qz(A,B);            % Schur decomposition
[S,T,Q,Z] = qzdiv(1,S,T,Q,Z);   % reorder of generalized eigenvalues 

Q1      = Q(1:nb,:);
Q2      = Q(nb+1:nb+nf,:);
Z11     = Z(1:nb,1:nb);
Z12     = Z(1:nb,nb+1:nb+nf);
Z21     = Z(nb+1:nb+nf,1:nb);
Z22     = Z(nb+1:nb+nf,nb+1:nb+nf);

if rank(Z11)<nb;
    error('Invertibility condition violated')
end

Z11i    = Z11\eye(nb);
S11     = S(1:nb,1:nb);
S12     = S(1:nb,nb+1:nb+nf);
S22     = S(nb+1:nb+nf,nb+1:nb+nf);
S11i    = S11\eye(nb);
T11     = T(1:nb,1:nb);
T12     = T(1:nb,nb+1:nb+nf);
T22     = T(nb+1:nb+nf,nb+1:nb+nf);

if abs(T(nb,nb))>abs(S(nb,nb)) | abs(T(nb+1,nb+1))<abs(S(nb+1,nb+1));
   warning('Wrong number of stable eigenvalues.');
end
TMPG    = Q2*C;
GAMMA   = (kron(Phi',S22)-kron(eye(nz),T22))\TMPG(:);
GAMMA   = reshape(GAMMA,nf,nz);
Fx      = real(Z21*Z11i);
Fz      = real((Z22-Z21*Z11i*Z12)*GAMMA);
Mx      = real(Z11*S11i*T11*Z11i);
Mz      = real(Z11*S11i*(T12*GAMMA-S12*GAMMA*Phi+Q1*C)-Mx*Z12*GAMMA+Z12*GAMMA*Phi);
Px      = real(TMPx1+TMPx2*Fx);
Pz      = real(TMPz+TMPx2*Fz);
sol.mx  = Mx;
sol.mz  = Mz;
sol.px  = Px;
sol.pz  = Pz;
sol.fx  = Fx;
sol.fz  = Fz;
sol.me  = Me;
sol.phi = Phi;
sol.sig = model.sig;
