%function [MY,ME,MU_]=singsolve(A0,A1,B,C)
%
% Paramtres structurels
%
alpha	= 0.4;
delta	= 0.025;
psi	= 0;
PSI	= 2.4631;
beta	= 0.9880;
rho	= 0.9500;
se		= 0.0070;
hs		= 1/3;
as		= 1;
%
% Etat stationnaire dterministe
%
ysk	= (1-beta*(1-delta))/(alpha*beta);
ksy	= 1/ysk;
isy	= delta*ksy;
csy	= 1-isy;

A0=zeros(4);
A0(1,1)=1;
A0(1,2)=-delta/(alpha*isy);
A0(1,3)=delta*(1-alpha*isy)/(alpha*isy);
A0(2,2)=1;
A0(3,3)=1;
A0(4,2)=-(1-beta*(1-delta))*rho/alpha;
A0(4,3)=-1;
A0(4,4)=(1-beta*(1-alpha)*(1-delta))/alpha;

A1=zeros(4);
A1(1,1)=(delta+isy*(1-delta))/isy;
A1(2,2)=rho;
A1(3,4)=1;

B=zeros(4,1);
B(2)=1;

C=zeros(4,1);
C(3)=4;

%
% [PI,MSS,MSE,MU]=singsolve(Mcc,Mcs,Mss0,Mss1,Msc0,Msc1,Mse,bkth_,shth_,fwth_,stth_,tol_,print_)
%
% Solves system of the form :
%
% A0 Et X(t+1) = A1 X(t)  + A2 e(t+1)
%
% where A0 can be singular.
%
% Deals with indeterminacy
%
% bkth_  : vector containing the index of endogenous backward variables in the system
% shth_  : vector containing the index of exogenous backward variables in the system
% fwth_  : vector containing the index of forward variables in the system
% stth_  : vector containing the index of static variables in the system
%
% tol_   : tolerance parameter to determine static variables (1e-8 by default)
% print_ : when print_=1 (by default) prints some diagnostic information, else does not print anything
%
% Returns a solution of the form :
%
% Y(t)   = PI S(t)
% F(t)   = GAM S(t)
% S(t+1) = MSS S(t) + MSE e(t+1)
%
% MU contains the generalized eigenvalues of the system.
% 
% (c) F. Collard, August 1999  This routine is an adaptation of routines by P. Sderlind.
%

% if (nargin<8)|isempty(tol_);
   tol_	= 1e-8;
% end;

% if (nargin<9)|isempty(print_);
   print_=1;
% end;
%
% First we compute the Schur decomposition A0=Q'*T*Z' and A1=Q'*S*Z'
%
[T S Q Z]	= qz(A0,A1);
[T S Q Z]   = qzdiv(1.0001,T,S,Q,Z);
%
% We recover the diagonal of the S and T matrix in order to 
%
%	1) Determine the status of variables
%	2) compute the generalized eigenvalues
%
DS_			= diag(S);
DT_			= diag(T);
%
% Dynamic variables 
%
dyn_		= find(abs(DT_)>tol_);	    % Determine dynamic variables
MU_			= DS_(dyn_)./DT_(dyn_);		% Compute generalized eigenvalues
AMU_		= abs(MU_);				    % Compute their modulus
m			= sum(AMU_>1+tol_);			% Find the forward variables
Q1          = Q(1:n-m,:);
Q2          = Q(n-m+1:n,:);
Z1          = Z(:,1:n-m)';
Z2          = Z(:,n-m+1:n)';

[U1,D1,V1]     = svd(Q1*C);
r1           = rank(Q1*C);
[U2,D2,V2]     = svd(Q2*C);
r2           = rank(Q2*C);


Phi         = ((U2(:,1:r2)*(D2(1:r2,1:r2)\V2(:,1:r2)'))*(V1(:,1:r)*D1(1:r,1:r)*U1(:,1:r)'))';
T11         = T(1:n-m,1:n-m);
T12         = T(1:n-m,n-m+1:n);
T22         = T(n-m+1:n,n-m+1:n);

T1i         = inv(T11);
S11         = S(1:n-m,1:n-m);
S12         = T(1:n-m,n-m+1:n);
S22         = T(n-m+1:n,n-m+1:n);

H           = Z*[T1i -T1i*(T12-Phi*T22);zeros(m,n-m) eye(m)];
MY          = Z*[T1i*S11 -T1i*(S12-Phi*S22);zeros(m,n)]*Z';
%MY          = Z*[T1i*S11 zeros(n-m,m);zeros(m,n)]*Z';
ME          = H*[(Q1-Phi*Q2);zeros(m,size(Q1,2))]*B;
