%function [MY,ME,MU_]=singsolve(A0,A1,B,C)
%
% Structural parameters
%
theta= 1;0.58;
whsy	= 0.6;0.7;
phi	= whsy/theta;
qksy	= 0.4;0.23;
alpha	= qksy/theta;
delta	= 0.025;
eta	= 0;
rho=0.95;
beta	= 0.988;0.99;
hs		= 0.31;
sz		= 0.00217;
%
% Deterministic Steady state
%
ysk	= (1-beta*(1-delta))/(alpha*beta*theta);
ksy	= 1/ysk;
isy	= delta/ysk;
csy	= 1-isy;
s=isy;

F0=zeros(4);
F0(1,1)=1;
F0(1,2)=-phi;
F0(2,1)=-1;
F0(2,2)=1;
F0(2,3)=1;
F0(3,3)=1;
F0(4,1)=1;
F0(4,3)=-(1-s);
F0(4,4)=-s;

F1=zeros(4,3);
F1(1,1)=alpha;
F1(1,2)=rho;
F1(3,3)=1;

F2=zeros(4,1);
F2(1)=1;

F3=zeros(4,1);
F3(3)=1;

PIY=inv(F0)*F1;
PIE=inv(F0)*F2;
PIZ=inv(F0)*F3;

A0Y=zeros(3);
A0Y(1,1)=1;
A0Y(2,2)=1;
A0Y(3,1)=1-beta*(1-delta);
A0Y(3,3)=1;

A0X=zeros(3,4);
A0X(3,1)=-(1-beta*(1-delta));

A1Y=zeros(3);
A1Y(1,1)=1-delta;
A1Y(2,2)=rho;

A1X=zeros(3,4);
A1X(1,4)=delta;
A1X(3,3)=1;

BY=zeros(3,1);
BY(2)=1;
CY=zeros(3,1);

A0=A0Y+A0X*PIY;
A1=A1Y+A1X*PIY;
B=BY+A1X*PIE;
C=CY+A1X*PIZ;

%
% First we compute the Schur decomposition A0=Q'*T*Z' and A1=Q'*S*Z'
%
tol_=1e-8;
[T S Q Z]	= qz(A0,A1);
[T S Q Z]   = qzdiv(1.0001,T,S,Q,Z);
n           = size(A0,1);   % number of variables
nf          = size(B,2);    % number of fundamental shocks
ne          = size(C,2);    % number of expectation errors
%
% We recover the diagonal of the S and T matrix in order to 
%
%	1) Determine the status of variables
%	2) compute the generalized eigenvalues
%
DS_			= diag(S);
DT_			= diag(T);
%
% Dynamic variables 
%
dyn_		= find(abs(DT_)>tol_);	    % Determine dynamic variables
MU_			= DS_(dyn_)./DT_(dyn_);		% Compute generalized eigenvalues
AMU_		= abs(MU_);				    % Compute their modulus
m			= sum(AMU_>1+tol_);			% Find the forward variables
Q1          = Q(1:n-m,:);
Q2          = Q(n-m+1:n,:);
Z1          = Z(:,1:n-m)';
Z2          = Z(:,n-m+1:n)';

[U1,D1,V1]  = svd(Q2*C);
r           = rank(Q2*C);               % number of linearly independent forecast errors

T11         = T(1:n-m,1:n-m);
T12         = T(1:n-m,n-m+1:n);
T22         = T(n-m+1:n,n-m+1:n);

T1i         = inv(T11);
S11         = S(1:n-m,1:n-m);
S12         = T(1:n-m,n-m+1:n);
S22         = T(n-m+1:n,n-m+1:n);

%Phi         = Q1*C*(U1(:,1:r)*(D1(1:r,1:r)\V1(:,1:r)'))';
[U1,D1,V1]   = svd(Q1*C);
r1           = rank(Q1*C);
[U2,D2,V2]   = svd(Q2*C);
r2           = rank(Q2*C);

Phi         = ((U2(:,1:r2)*(D2(1:r2,1:r2)\V2(:,1:r2)'))*(V1(:,1:r1)*D1(1:r1,1:r1)*U1(:,1:r1)'))';


Transfo     = [eye(n-m) -Phi];
MY0         = inv([Transfo*T;zeros(m,n-m) eye(m)]);
MY1         = ([Transfo*S;zeros(m,n)]);
MY          = Z*(MY0*MY1)*Z';
if ne>r;
    fprintf('I will add %d sunspot variables\n',ne-r)
    ME          = MY0*[Transfo*Q*[B C];zeros(m,size([B C],2))];
    ME          = Z*ME;
else
    ME          = MY0*[Transfo*Q*B;zeros(m,size(B,2))];
    ME          = Z*ME;
end

%[PIE PIZ]-PIZ*(U1(:,1:r)*(D1(1:r,1:r)\V1(:,1:r)'))'*Q2*[B C]

%PIZ*(eye(r)-V1*V1')