clear all   % Clear memory
%
% Structural parameters
%
alpha	= 0.4;
gy      = 0.25;
gp      = 0.5;
rho_g   = 0.9;
rho_u   = 0.95;
lambda  = 1;
beta    = 0.9;
% Define:
%
% Y=[y(t),pi(t),g(t),u(t),E_t y(t+1),E_t pi(t+1)]
%
ny  = 6; % # of variables in vector Y
ne  = 2; % # of fundamental shocks
nn  = 2; % # of expectation errors
%
% Initialize the matrices
%
A0  = zeros(ny,ny);
A1  = zeros(ny,ny);
B   = zeros(ny,ne);
C   = zeros(ny,nn);
%
% Output
%
A0(1,1) = 1;
A1(1,5) = 1;
C(1,1)  = 1;
%
% Inflation
%
A0(2,2) = 1;
A1(2,6) = 1;
C(2,2)  = 1;
%
% IS shock
%
A0(3,3) = 1;
A1(3,3) = rho_g;
B(3,1)  = 1;
%
% Supply shock
%
A0(4,4) = 1;
A1(4,4) = rho_u;
B(4,2)  = 1;
%
% IS curve
%
A0(5,1) = 1+alpha*gy;
A0(5,2) = alpha*gp;
A0(5,3) = -1;
A0(5,5) = -1;
A0(5,6) = -alpha;
%
% Phillips Curve
%
A0(6,1) = -lambda;
A0(6,2) = 1;
A0(6,4) = -1;
A0(6,6) = -beta;
%
% Call Sims' routine
%
[MY,ME,ETA,MU_]=sims_solve(A0,A1,B,C);
break
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                            %
%               Impulse Response Functions                   %
%                                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nrep    = 20;            % horizon of responses
YS      = zeros(3,nrep);
XS      = zeros(4,nrep);
Shock   = 1;
YS(:,1) = ME*Shock;
XS(:,1) = PIE;
for t=2:nrep;
    YS(:,t) = MY*YS(:,t-1);
    XS(:,t) = PIY*YS(:,t-1);
end
subplot(221);plot(XS(1,:));title('Output');xlabel('Time')
subplot(222);plot(XS(2,:));title('Consumption');xlabel('Time')
subplot(223);plot(XS(3,:));title('Investment');xlabel('Time')
subplot(224);plot(XS(4,:));title('Hours worked');xlabel('Time')