clear;
%
% Structural parameters
%
theta= 0.58;
whsy	= 0.7;
phi	= whsy/theta;
qksy	= 0.23;
alpha	= qksy/theta;
delta	= 0.025;
eta	= 0;
beta	= 0.99;
hs		= 0.31;
sz		= 0.00217;
%
% Deterministic Steady state
%
ysk	= (1-beta*(1-delta))/(alpha*beta*theta);
ksy	= 1/ysk;
isy	= delta/ysk;
csy	= 1-isy;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                 Parameters of the algorithm                 %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ncont	= 3;  	            		% # of measurement equations
nbend	= 1;               		% # of endogenous backward variables
nshoc	= 0;               		% # of shocks
nback	= nbend+nshoc;				% # of backward variables
nforw	= 1;               		% # of forward variables
nstat	= nback+nforw;     		% # of state equations
%
% Initializing matrices
%
Mcc	= zeros(ncont,ncont);		
Mcs	= zeros(ncont,nstat);		
Mss0	= zeros(nstat,nstat);	
Mss1	= zeros(nstat,nstat);	
Msc0	= zeros(nstat,ncont);	
Msc1	= zeros(nstat,ncont);	
Mse	= zeros(nstat,nshoc);		
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                      matrix coefficients                    %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% h y i
% 1 2 3 
%
% k c
% 1 2  
%
% Hours
%
Mcc(1,1)	= (1+eta);
Mcc(1,2)	= -1;
Mcs(1,2)	= -1;
%
% Output
%
Mcc(2,1)	= -phi;
Mcc(2,2)	= 1;
Mcs(2,1)	= alpha;
%
% Investment
%
Mcc(3,2)	= 1;
Mcc(3,3)	= -isy;
Mcs(3,2)	= csy;
%
% Capital
%
Mss0(1,1)	= 1;
Mss1(1,1)	= delta-1;
Msc1(1,3)	= delta;
%
% Consumption
%
Mss0(2,1)	= (1-beta*(1-delta));
Mss0(2,2)	= 1;
Mss1(2,2)	= -1;
Msc0(2,2)	= (1-beta*(1-delta));

bkth_=[1];
shth_=[];
fwth_=[2];
stth_=[];
tol_=1e-6;
print_=1;
%
% Solve the system with the singular method
%
[PI,GAM,MSS,MSE,MSZ,MU_]=singsolve(Mcc,Mcs,Mss0,Mss1,Msc0,Msc1,Mse,bkth_,shth_,fwth_,stth_,tol_,print_);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                  Impulse Response Functions                 %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
nrep		= 80;
ST			= zeros(size(MSS,1),nrep);
SHOCK		= 1;
ST(:,1)	= MSZ*SHOCK;
for i		= 2:nrep;
   ST(:,i)	= MSS*ST(:,i-1);
end
YT			= (PI*ST)';
FT			= (GAM*ST)';
ST			= ST';

subplot(221);plot(YT(:,1));
title('Hours','fontname','times','fontsize',12);
xlabel('Quarters','fontname','times','fontsize',12);
set(gca,'fontname','times','fontsize',12);
subplot(222);plot(YT(:,2));
title('Output','fontname','times','fontsize',12);
xlabel('Quarters','fontname','times','fontsize',12);
set(gca,'fontname','times','fontsize',12);
subplot(223);plot(ST(:,2));
title('Consumption','fontname','times','fontsize',12);
xlabel('Quarters','fontname','times','fontsize',12);
set(gca,'fontname','times','fontsize',12);
subplot(224);plot(YT(:,3));
title('Investment','fontname','times','fontsize',12);
xlabel('Quarters','fontname','times','fontsize',12);
set(gca,'fontname','times','fontsize',12);
pause;
%print -depsc2 d:\texwork\deeqa\chapter2\irf_fg94.eps
close
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                             %
%                      Simulated data                         %
%                                                             %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
long	= 120;
init	= 100;
slong	= long+init;
T		= init+1:slong;
nsim	= 100;
SX=0;
SX2=0;
CX=0;
CX2=0;
for j=1:nsim;
   disp(sprintf('Simulation : %4.0f',j));
   randn('state',j);
   CHOC			= randn(1,slong);
   CHOC			= sz*(CHOC-mean(CHOC))/std(CHOC);    
   STS			= zeros(size(MSS,1),slong);
   STS(:,1)	= MSZ*CHOC(:,1);
   for i		= 2:slong;
		STS(:,i)= MSS*STS(:,i-1)+MSZ*CHOC(:,i);
   end;
   YTS			= (PI*STS(:,T))';
   FTS			= (GAM*STS(:,T))';
   STS			= STS(:,T)';
   X				= [YTS STS(:,2)];
   tmp			= std(X);
   SX				= SX+tmp/nsim;
   SX2			= SX2+tmp.*tmp/nsim;
   tmp			= corrcoef(X);
   CX				= CX+tmp/nsim;
   CX2			= CX2+tmp.*tmp/nsim;
end;                                                         
[100*SX;100*(SX2-SX.*SX).^0.5]
[CX(2,:);real((CX2(2,:)-CX(2,:).*CX(2,:)).^0.5)]